/*
 *
 * Copyright (c) 1998 Scott Oaks. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.io.*;
import java.security.*;
import java.util.*;

public class XYZFileScope extends IdentityScope {
	private Hashtable ids;
	private static String fname;

	/*public final static String PrivateScopeName = "/floppy/floppy0/private";
	public final static String PrivateScopeName = "/tmp/privateScope";
	public final static String PublicScopeName = "sharedScope";*/

	public XYZFileScope(String fname) throws KeyManagementException {
		super("XYZFileScope");
		this.fname = fname;
		try {
			FileInputStream fis = new FileInputStream(fname);
			ObjectInputStream ois = new ObjectInputStream(fis);
			ids = (Hashtable) ois.readObject();
		} catch (FileNotFoundException fnfe) {
			ids = new Hashtable();
		} catch (Exception e) {
			System.out.println(e);
			throw new KeyManagementException("Can't load identity database " + fname);
		}
	}
	
	public int size() {
		return ids.size();
	}

	public Identity getIdentity(String name) {
		Identity id;
		id = (Identity) ids.get(name);
		return id;
	}

	public Identity getIdentity(PublicKey key) {
		if (key == null)
			return null;
		Identity id;
		for (Enumeration e = ids.elements(); e.hasMoreElements(); ) {
			id = (Identity) e.nextElement();
			PublicKey k = id.getPublicKey();
			if (k != null && k.equals(key))
				return id;
		}
		return null;
	}

	public void addIdentity(Identity identity) throws KeyManagementException {
		String name = identity.getName();
		if (getIdentity(name) != null)
			throw new KeyManagementException(name + " already in identity scope");

		PublicKey k = identity.getPublicKey();
		if (getIdentity(k) != null)
			throw new KeyManagementException(name + " already in identity scope");
		ids.put(name, identity);
	}

	public void removeIdentity(Identity identity) throws KeyManagementException {
		String name = identity.getName();
		if (ids.get(name) == null)
			throw new KeyManagementException(name + " isn't in the identity scope");
		ids.remove(name);
	}

	public Enumeration identities() {
		return ids.elements();
	}

	public void save() {
		try {
			FileOutputStream fos = new FileOutputStream(fname);
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(ids);
		} catch (Exception e) {
			System.out.println(e);
			throw new RuntimeException("Can't save id database");
		}
	}
}
