/*
 *
 * Copyright (c) 1998 Scott Oaks. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.security.*;
import java.util.*;

public class XYZPayrollPermissionCollection extends PermissionCollection {
	private Hashtable permissions;
	private boolean addedAdmin;
	private int adminMask;

	XYZPayrollPermissionCollection() {
		permissions = new Hashtable();
		addedAdmin = false;
	}

	public void add(Permission p) {
		if (!(p instanceof XYZPayrollPermission))
			throw new IllegalArgumentException("Wrong permission type");
		XYZPayrollPermission xyz = (XYZPayrollPermission) p;
		String name = xyz.getName();
		XYZPayrollPermission other = (XYZPayrollPermission) permissions.get(name);
		if (other != null)
			xyz = merge(xyz, other);
		if (name.equals("*")) {
			addedAdmin = true;
			adminMask = xyz.mask;
		}
		permissions.put(name, xyz);
	}

	public Enumeration elements() {
		return permissions.elements();
	}

	public boolean implies(Permission p) {
		if (!(p instanceof XYZPayrollPermission))
			return false;
		XYZPayrollPermission xyz = (XYZPayrollPermission) p;
		if (addedAdmin && (adminMask & xyz.mask) != 0)
			return true;
		Permission inTable = (Permission) permissions.get(xyz.getName());
		if (inTable == null)
			return false;
		return inTable.implies(xyz);
	}

	private XYZPayrollPermission merge(XYZPayrollPermission a, XYZPayrollPermission b) {
		String aAction = a.getActions();
		if (aAction.equals(""))
			return b;
		String bAction = b.getActions();
		if (bAction.equals(""))
			return a;
		return new XYZPayrollPermission(a.getName(), aAction + "," + bAction);
	}
}
